import os
import re
import hashlib
import subprocess

class Dsc:
    SECTION_TO_HASH_FUNC = {
        'Checksums-Sha1:':   hashlib.sha1,
        'Checksums-Sha256:': hashlib.sha256,
        'Files:':            hashlib.md5,
    }

    def __init__(self, filename):
        """
        @param string filename the dsc filename
        """
        self.__filename = filename

    def init(self):
        with open(self.__filename, 'r') as handle:
            self.__dsc = handle.read()

        return self

    def write(self):
        with open(self.__filename, 'w') as handle:
            handle.write(self.__dsc)

        return self

    # TODO: Sanity check that the version includes the suffix +dattoX

    def updateShas(self):
        """
        Update the shas within the debian source control file.
        """
        self.__dsc = '\n'.join(self.__processShaLines(self.__dsc.split('\n')))

        return self

    def stripGpg(self):
        """
        Remove the GPG header from the debian source control file.-
        """
        process = subprocess.Popen(['gpg', '-o-', '-'], stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

        process.stdin.write(self.__dsc)

        output, error = process.communicate()

        # gpg is always returning an exit code of 2 for me, which means no error checking :-(
        #assert process.returncode == 0, 'Unable to strip GPG header'

        # because we can't check the exit code, if gpg fails, we'll just leave the __dsc file alone
        if len(output) > 0:
            self.__dsc = output

        return self

    def __processShaLines(self, inputLines):
        """
        Iterate over the lines within the dsc file and update the file shas and byte counts.
        """
        outputLines = []
        hashFunction = None
        for line in inputLines:
            # e.g.: 1a546a1a2ed3a39d49ada99821db401a 1234878 hivex_1.3.9.orig.tar.gz
            match = re.search(' (\w+) (\d+) (.*)', line)

            if match == None or hashFunction == None:
                if line.strip() in Dsc.SECTION_TO_HASH_FUNC:
                    hashFunction = Dsc.SECTION_TO_HASH_FUNC[line.strip()]
                else:
                    hashFunction = None
            else:
                (hash, size, filename) = match.group(1, 2, 3)

                # If the file isn't downloaded, we're not going to recalculate the hash
                if os.path.isfile(filename):
                    newSize = os.path.getsize(filename)
                    with open(filename, 'rb') as f:
                        newHash = hashFunction(f.read()).hexdigest()
                    line = ' %s %d %s' % (newHash, newSize, filename)

            outputLines.append(line)
        return outputLines

    def getDebianSourceFilename(self):
        """
        Get the filename of the .debian.tar.xz
        """
        return self.__findDebianSourceFilename(self.__dsc.split('\n'))

    def __findDebianSourceFilename(self, inputLines):
        for line in inputLines:
            match = re.search(' \w+ \d+ (.*\.debian\..*)', line)

            if match != None:
                return match.group(1)

        return None
